package codelines;

import animation.CodeLine;
import animation.ControlFlow;
import animation.Memory;
import animation.StackFrame;
import animation.StackFrame.FrameType;

public class FunctionDefinition extends CodeLine {

	private String[] params;
	
	public FunctionDefinition( String[] params )
	{
		this.params = params;
	}
	
	@Override
	public ControlFlow runForward(Memory m) {
		if( !m.isDefined( "param1", true ) && !m.isDefined( "line_" + lineId + "_inside", false ) )
			return new ControlFlow( ControlFlow.STEP_OVER );
		if( !m.isDefined( "line_" + lineId + "_inside", false ) )
		{
			m.addFrame( new StackFrame( FrameType.FUNCTION ) );
			m.declare( "line_" + lineId + "_inside", true, false );
			int index = 1;
			Object olds[] = new Object[ params.length ];
			for( String p : params )
			{
				olds[ index - 1 ] = m.read( "param" + index, true );
				m.declare( p, olds[ index - 1 ], false );
				m.undeclare( "param" + index, true );
				index++;
			}
			actions.push( (Memory mem) -> {
				mem.removeFrame();
				int i = 1;
				for( @SuppressWarnings("unused") String p : params )
				{
					mem.declare( "param" + i, olds[ i - 1], true );
					i++;
				}
				return new ControlFlow( ControlFlow.STEP_OVER );
			} );
			return new ControlFlow( ControlFlow.STEP_INTO );
		}
		else
		{
			StackFrame frame = m.removeFrame();
			actions.push( (Memory mem) -> {
				mem.addFrame( frame );
				return new ControlFlow( ControlFlow.STEP_INTO ); // call function backwards
			} );
			return new ControlFlow( ControlFlow.STEP_OVER ); // return
		}
	}
}
